% -------------Added by caicai-------------
%% A demo for learning Linear Discriminant Projections (LDP) with simulating
%% strategy
%
%% demo_learn_projections.m
% 
%% Reference: (NOTE: U used in the codes is W in the paper)
%  Hongping Cai, Krystian Mikolajczyk, and Jiri Matas. Learning linear discriminant projections for dimensionality reduction of image descriptors.
%  In BMVC 2008: Proceedings of the 19th British Machine Vision Conference, vol. 1, pages 503-512, Leeds, UK, September 2008. 
%
%% Authors:
%  Hongping Cai         (hongpingcai@hotmail.com), 
%  Krystian Mikolajczyk (k.mikolajczyk@surrey.ac.uk)
%%
% ----------- 14th, Nov, 2009 --------------

%% parameters setting
rand('twister', sum(100*clock));
detector = 'dense';
descriptor = 'sift';
sim_para.type   = {'rotate','scale','translate'}; %% three types of transforms
sim_para.factor = [9.360 1.128 1.952];            %% the standard deviation of Gaussion/Uniform distribution for each transform
sim_para.num    = 3;                              %% 3 simulated matched features for each transform and each region 
delta = [8 14 20 26];                             %% 4 types of dense sampling intervals
r = [16 24 32 40];                                %% 4 types of radii, corresponded to the intervals

nimg = 3;
%% STEP1: dense sampling
disp('STEP1: dense sampling...');
for i=1:nimg
    img_file    = ['data/image_000' int2str(i) '.png'];                  
    detect_file = ['data/image_000' int2str(i) '.' detector];
    detect_dense_regions(img_file,delta,r,detect_file);
    disp(['    img' int2str(i) ': Done. saved in ' detect_file]);
end;
    
%% STEP2: extract SIFT features
disp('STEP2: extract SIFT features...');
for i=1:nimg
    img_file    = ['data/image_000' int2str(i) '.png'];   %% only png,ppm images are supported       
    detect_file = ['data/image_000' int2str(i) '.' detector];
    sift_file   = ['data/image_000' int2str(i) '.' detector '.' descriptor];
    tmp_outfile = ['data/tmpout' num2str(rand(1)) '_desc_vidi_64_angle.txt'];
    command = sprintf('./desc_vidi_64_angles.ln %d %s %s %s>%s',0,img_file,detect_file,sift_file,tmp_outfile);
    system(command);
    delete(tmp_outfile);
    disp(['    img' int2str(i) ': Done. saved in ' sift_file]);
end;

all_feat = [];
all_diff = [];
%% STEP3: simulate matched features
disp('STEP3: simulate matched features...');
for i=1:nimg
    fprintf(1,'    img%d: \n',i);
    img_file    = ['data/image_000' int2str(i) '.png'];   %% only png,ppm images are supported       
    sift_file   = ['data/image_000' int2str(i) '.' detector '.' descriptor];
    [feat n_rg sift_dim]=loadFeatures(sift_file);
    feat = feat';
    feat_loc = feat(:,1:2);
    [feat_loc_uniq,ind_uniq,tmpm] = unique(feat_loc,'rows');
    feat_uniq = feat(ind_uniq,:);  %%No two regions locate the same position!!

    all_feat = [all_feat; feat_uniq(:,6:end)];
    b_inside = 1;%% only inside regions are considered, i.e., not on the boundary of the images
    flag = 0;    %% uniform distributed transformations
    tmp_file1 = ['data/tmp' num2str(rand(1)) '.' detector];
    tmp_file2 = ['data/tmp' num2str(rand(1)) '.' detector '.' descriptor];
    fprintf(1,'    simulate matched features with translation changes...');
    [diff_feat] = diff_translate(img_file,feat_uniq,sim_para.factor(3),sim_para.num,...
        tmp_file1,tmp_file2,b_inside,flag);
    all_diff = [all_diff; diff_feat];
    fprintf(1,'    simulate matched features with scale changes...');
    [diff_feat] = diff_scale(img_file,feat_uniq,sim_para.factor(2),sim_para.num,...
        tmp_file1,tmp_file2,b_inside,flag);
    all_diff = [all_diff; diff_feat];
    fprintf(1,'    simulate matched features with rotation changes...');
    [diff_feat] = diff_rotate(img_file,feat_uniq,sim_para.factor(1),sim_para.num,...
        tmp_file1,tmp_file2,b_inside,flag);
    all_diff = [all_diff; diff_feat];
    delete(tmp_file1);
    delete(tmp_file2);
    delete([tmp_file2 '.angle']);
end;
fprintf(1,'    Done. %d unmatch features, %d matched feature pairs\n',size(all_feat,1),size(all_diff,1));

%% STEP4: compute the projections (LDP(P), U, PCA(V))
disp('STEP4: compute the projections (P, U, V(pca))...');
cov_unmatched = cov(all_feat); 
cov_matched   = cov(all_diff);
mean_feat = mean(all_feat,1);
[P,U,V] =compute_PUV(cov_unmatched,cov_matched);
puv_file = ['data/PUV.mat'];
save(puv_file,'P','U','V','mean_feat');
disp(['    Done. The projections are saved in ' puv_file]);

%% show the histogram for unmatched and matched feature distances
%% show the intra-class and inter-class covariances
%% show the three projections
b_show = 1;
if b_show
    figure;
    set(gcf,'defaultaxesfontsize',12);
    e = all_diff.^2;
    ee = sum(e');
    ee = sqrt(ee);
    subplot(2,2,1);
    hist(ee,20);
    title('simulated matched feature distance');
    ii = floor(rand(size(all_diff,1),1).*(size(all_feat,1)-1))+1;
    jj = floor(rand(size(all_diff,1),1).*(size(all_feat,1)-1))+1;    
    ind = find(ii~=jj);   
    e = (all_feat(ii(ind),6:end) - all_feat(jj(ind),6:end)).^2;
    ee = sum(e');
    ee = sqrt(ee);
    subplot(2,2,2);
    hist(ee,20);
    title('unmatched feature distance');
    subplot(2,2,3);
    imshow(cov_matched,[]);
    title('intra-class covariance');
    subplot(2,2,4);
    imshow(cov_unmatched,[]);
    title('inter-class covariance');

    dim = 128;    
    figure;
    set(gcf,'defaultaxesfontsize',12);
    for i=1:5
        subplot(5,3,3*i-2);
        plot(P(i,:),'Linewidth',1.5);
        xlim([1 dim]);
        title(['P(' int2str(i) ')']);
        subplot(5,3,3*i-1);
        plot(U(i,:),'Linewidth',1.5);
        xlim([1 dim]);
        title(['U(' int2str(i) ')']);
        subplot(5,3,3*i);
        plot(V(i,:),'Linewidth',1.5);
        xlim([1 dim]);
        title(['PCA(' int2str(i) ')']);
    end
end;

%% help for desc_vidi_64_angles.ln (compiled on a 64-bit Linux machine)
%
%  flag      -- input angle's option
%               flag=0: all angles are 0, one angle per region (suggested for dense sampled regions)
%               flag=1: all angles are dominant angles, one angle per region
%               flag=2: all angles are dominant angles, mutiple angles permitted
%  img_file  -- input image file name, only *.png and *.ppm images are supported
% detect_file-- input region file
% sift_file  -- output sift feature file ( and also an angle-scale file,
%               named [sift_file '.angle'],  will be companied. In the
%               angle-scale file, two columns correspond to the dominant
%               angles and scales.
%
%  <<detect_file format>>
% 1
% n
% x_1 y_1 a_1 b_1 c_1
%      :
%      :
% x_n y_1 a_n b_n c_n
%
%  <<sift_file and projected feature file format>>
% dim
% n
% x_1 y_1 a_1 b_1 c_1 f_1_1 .. f_1_dim
%      :
%      :
% x_n y_1 a_n b_n c_n f_n_1 .. f_n_dim
%
% x,y,a,b,c are 5 parameters for an ellipse:  a(x-u)(x-u)+2b(x-u)(y-v)+c(y-v)(y-v)=1
% For a circle with radius r, a = 1/r^2, b = 0, c = 1/r^2.



            
            