% -------------Added by caicai-------------
%% A demo for generating sift features and projecting them with LDP(P), U and
%% PCA(V) projections
%
%% demo_project_feature.m
% 
%% Reference: (NOTE: U used in the codes is W in the paper)
%  Hongping Cai, Krystian Mikolajczyk, and Jiri Matas. Learning linear discriminant projections for dimensionality reduction of image descriptors.
%  In BMVC 2008: Proceedings of the 19th British Machine Vision Conference, vol. 1, pages 503-512, Leeds, UK, September 2008. 
%% Author:
%  Hongping Cai         (hongpingcai@hotmail.com), 
%  Krystian Mikolajczyk (k.mikolajczyk@surrey.ac.uk)
%%
% ----------- 14th, Nov, 2009 --------------

%% parameters setting
detector = 'dense';
descriptor = 'sift';
flag = 0;
delta = [8 14 20 26];
r = [16 24 32 40];
img_file    = 'data/image_0001.png';  %% only png,ppm images are supported
detect_file = ['data/image_0001.' detector]; 
sift_file   = ['data/image_0001.' detector '.' descriptor];
dim = 30;     %% the projected feature dimension
P_feat_file = ['data/P' int2str(dim) '_image_0001.' detector '.' descriptor];  %% P-projected
U_feat_file = ['data/U' int2str(dim) '_image_0001.' detector '.' descriptor];  %% U-projected
V_feat_file = ['data/V' int2str(dim) '_image_0001.' detector '.' descriptor];  %% PCA-projected

%% STEP1: dense sampling
disp('STEP1: dense sampling...');
detect_dense_regions(img_file,delta,r,detect_file);
disp(['     Done. saved in ' detect_file]);

%% STEP2: extract SIFT features
disp('STEP2: extract SIFT features...');
rand('twister', sum(100*clock));
tmp_outfile = ['data/tmpout' num2str(rand(1)) '_desc_vidi_64_angle.txt'];
command = sprintf('./desc_vidi_64_angles.ln %d %s %s %s>%s',flag,img_file,detect_file,sift_file,tmp_outfile);
system(command);
delete(tmp_outfile);
disp(['     Done. saved in ' sift_file]);

%% STEP3: load projections (LDP(P), U, PCA)
disp('STEP3: load projections (LDP(P), U, PCA)...');
project_file = 'data/PUV.mat';%or 'pascal07_projections.mat','caltech101_projections1.mat'
if ~exist(project_file)
    error('No projection file! Please generate it by running demo_learn_projections.m, or load our provided projections on Caltech101 and PASCAL VOC2007!');
else
    load(project_file,'P','U','V','mean_feat'); % each row corresponds to one vector in P, U, V
end;

%% STEP4: project features into lower dimensions and normalize them
disp('STEP4: project features into lower dimensions and normalize them...');
[feat n_rg sift_dim]=loadFeatures(sift_file);
pure_feat = feat(6:end,:)';
pure_feat = bsxfun(@minus, pure_feat, mean_feat);  % substract the mean first
pure_feat = pure_feat';
% LDP (or P)
P_feat = P(1:dim,:)*pure_feat;
[norm_feat] = norm_vectors(P_feat');
norm_feat = [feat(1:5,:)' norm_feat];
writeFeatures(norm_feat,P_feat_file);
disp(['    Done. The LDP(P)-projected ' int2str(dim) '-dim feature file is saved in ' P_feat_file ]);
% U
U_feat = U(1:dim,:)*pure_feat;
[norm_feat] = norm_vectors(U_feat');
norm_feat = [feat(1:5,:)' norm_feat];
writeFeatures(norm_feat,U_feat_file);
disp(['    Done. The U-projected ' int2str(dim) '-dim feature file is saved in ' U_feat_file ]);
% PCA (or V)
V_feat = V(1:dim,:)*pure_feat;
[norm_feat] = norm_vectors(V_feat');
norm_feat = [feat(1:5,:)' norm_feat];
writeFeatures(norm_feat,V_feat_file);
disp(['    Done. The PCA(V)-projected ' int2str(dim) '-dim feature file is saved in ' V_feat_file]);


%% help for desc_vidi_64_angles.ln (compiled on a 64-bit Linux machine)
%
%  flag      -- input angle's option
%               flag=0: all angles are 0, one angle per region (suggested for dense sampled regions)
%               flag=1: all angles are dominant angles, one angle per region
%               flag=2: all angles are dominant angles, mutiple angles permitted
%  img_file  -- input image file name, only *.png and *.ppm images are supported
% detect_file-- input region file
% sift_file  -- output sift feature file ( and also an angle-scale file,
%               named [sift_file '.angle'],  will be companied. In the
%               angle-scale file, two columns correspond to the dominant
%               angles and scales.
%
%  <<detect_file format>>
% 1
% n
% x_1 y_1 a_1 b_1 c_1
%      :
%      :
% x_n y_1 a_n b_n c_n
%
%  <<sift_file and projected feature file format>>
% dim
% n
% x_1 y_1 a_1 b_1 c_1 f_1_1 .. f_1_dim
%      :
%      :
% x_n y_1 a_n b_n c_n f_n_1 .. f_n_dim
%
% x,y,a,b,c are 5 parameters for an ellipse:  a(x-u)(x-u)+2b(x-u)(y-v)+c(y-v)(y-v)=1
% For a circle with radius r, a = 1/r^2, b = 0, c = 1/r^2.

